#!/usr/bin/perl 

use Getopt::Std;
$version="0.3 19/12/03";

$usage =<<EOM
-----------------------------------------------------------------------
roff.pl  (version $version)

  lee de un archivo con offset, tantos bytes como se quiera y los
  muestra en pantalla con el caracter ASCII o usando un formateo
  hexa

  Uso:
  roff.pl [-C] [-x] [-b] [-o offset] [-c count] [-s buffsize] archivo

  Parametros requeridos:
  archivo de lectura 

  Parametros opcionales:
  -C muestra un dump con el programa "hexdump"

  -x muestra la salida como numeros hexadecimales codificados
     en dos caracteres ASCII

  -b utiliza la lectura buffereada (read)
     por defecto se utiliza sysread, que no es buffereada

  -o byte de offset de inicio de la lectura.
     por defecto = 0

  -c cantidad de bytes a leer a partir del offset.
     por defecto = 128

  -s tama~o del buffer de lectura
     por defecto = 512

  Ejemplo:

  ./roff.pl -x -o 2 -c 30 /dev/rw_bar1
    lee 30 bytes a partir del byte 2 del archivo /dev/rw_bar1

----

  Proyecto IIE-PCI
  ciro\@mondueri.com / sebfer\@iie.edu.uy

-----------------------------------------------------------------------
EOM
;

# obtento parametrs de la entrada estandar
getopt('ocs');
getopts('bxC');
$dev = @ARGV[0];

unless ($ARGV[0]) {
    print $usage;
    exit(1);
}

$offset = $opt_o || 0;
$count = $opt_c || 128;
$buffsize = $opt_s || 512;

if ($opt_x) {
    # verifico si se esta usando UTF8 en la variable LANG
    if ($ENV{LANG} =~ /.*UTF.*/) {
	print "-----------------------------------------------------------------------\n";
	print "ADVERTENCIA: se esta utilizando UTF8 o UTF16 en la variable de entorno LANG.\nEsto puede hacer que el modo -x no funcione correctamente.\nSe recomienda cambiar la variable de entorno LANG=en_US\n-----------------------------------------------------------------------\n";
    }
}

my $out=0;

open(RW,$dev) || die "El archivo de entrada no existe\n";

if ($opt_o) {
    sysseek(RW,$offset, 1);
}

if ($opt_C) {
    $pid = open(WRITEME, "| hexdump -C");
}

my $totalread = 0;
my $buf;
if ($opt_b) {
    # lecutra buffereada
    while(read(RW, $buf, $buffsize) and ($totalread < $count)){
	if ($opt_C) {
	    print WRITEME $buf;
	} elsif ($opt_x) {
	    print unpack("H*",$buf);
	} else {
	    print $buf;
	}
	$totalread += length($buf);
    }
} else {
    # lectura sin buffer
    while(sysread(RW, $buf, $buffsize) and ($totalread < $count)){
	if ($opt_C) {
	    print WRITEME $buf;
	} elsif ($opt_x) {
	    print unpack("H*",$buf);
	} else {
	    print $buf;
	}
	$totalread += length($buf);
    }
}


if ($opt_C) {
    close(WRITEME);
}

close(RW);

