#!/usr/bin/perl

use Getopt::Std;
$version="0.3 19/12/03";

$usage =<<EOM
-----------------------------------------------------------------------
woff.pl  (version $version)

  Uso:
  woff.pl [-x] [-b] [-v] [-o offset] [-c count] [-s buffsize] archivo

  escribe en el archivo los datos que recibe desde la entrada estandar
  en la posicion offset y count bytes.

  NOTA: como echo -n no se puede utilizar, ya que la entrada estandar
        no es llenada, el programa siempre quita el enter final
        agregado por el echo. 

  NOTA: la variable de entorno LANG no debe estar configurada para 
        utilizar caracteres UTF8. Se recomienda LANG=en_US.

  Parametros requeridos:
  * archivo de lectura 
  * datos por entrada estandar


  Parametros opcionales
  -x interpreta la entrada estandard como numeros hexadecimales

  -b utiliza la escritura buffereada (print)
     por defecto se utiliza syswrite, que no es buffereada

  -v verbose mode

  -o byte de offset de inicio de la lectura.
     por defecto = 0

  -c cantidad de bytes a leer de la entrada
     (en modo -x es la cantidad de caracteres. 1 byte = 2 caracteres)
     por defecto = 16kB

  -s tama~o del buffer de lectura -> escritura
     por defecto = 512 bytes

  Ejemplos:

  echo "testing" | ./woff.pl -o 2 /dev/rw_bar1 
    escribe el string "testing" a partir del byte 2 en el
    archivo /dev/rw_bar1. automaticamente quita el enter agregado
     por echo.

  echo FFAA | ./woff.pl -x /dev/rw_bar1
    escribe los dos numeros hexa xFF y xAA a partir del
    archivo /dev/rw_bar1. 


----

  Proyecto IIE-PCI
  ciro\@mondueri.com / sebfer\@iie.edu.uy

-----------------------------------------------------------------------
EOM
;

# obtento parametrs de la entrada estandar
getopt('ocs');
getopts('bxv');

$dev = $ARGV[0];
my $offset = $opt_o || 0;
my $buffsize = $opt_s || 512;


unless ($ARGV[0]) {
    print $usage;
    exit(1);
}

if ($opt_x) {
    # verifico si se esta usando UTF8 en la variable LANG
    if ($ENV{LANG} =~ /.*UTF.*/) {
	print "-----------------------------------------------------------------------\n";
	print "ADVERTENCIA: se esta utilizando UTF8 o UTF16 en la variable de entorno LANG.\nEsto puede hacer que el modo -x no funcione correctamente.\nSe recomienda cambiar la variable de entorno LANG=en_US\n-----------------------------------------------------------------------\n";
    }
}

# obtengo la cantidad de bytes leer de la entrada
$count = $opt_c || 16*1024;

# abro el archivo de salida
open(RW,"+<$dev") || die "ERROR: el archivo de salida no existe\n";

# avanzo hasta el offset deseado
if ($opt_o) {
    sysseek(RW,$offset, 1);
}

my $totalwrite = 0;

while(read(STDIN, $buf, $buffsize) and ($totalwrite < $count) ){
    if ($opt_x) {
	# proceso la entrada como representacion ascii de hexa
	$writedata = pack("H*",$buf);
    } else {
	$writedata = $buf;
    }

    if ($opt_b) {
	# utilizo llamada de escritura buffereada
	$written = print RW $writedata;
    } else {
	# utilizo llamada de escritura no buffereada
	$written = syswrite(RW, $writedata, length($writedata));
    }

    $totalwrite += $written;
}

if ($opt_v) {
    print "Bytes escritos: $totalwrite\n";
}
close(RW);

